@@ -89,7 +89,7 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 #if _FFR_8BITENVADDR
 	if (bitset(RF_IS_EXT, flags) && addr != NULL)
 	{
-		int len = 0;
+		int len;
 
 		addr = quote_internal_chars(addr, NULL, &len, NULL);
 	}
@@ -157,7 +157,7 @@ parseaddr(addr, a, flags, delim, delimptr, e, isrcpt)
 #if _FFR_8BITENVADDR
 	if (NULL != a->q_user)
 	{
-		int len = 0;
+		int len;
 
 		a->q_user = quote_internal_chars(a->q_user, NULL, &len, e->e_rpool); /* EAI: ok */
 	}
@@ -2016,6 +2016,9 @@ buildaddr(tv, a, flags, e)
 	char **hostp;
 	char hbuf[MAXNAME + 1];	/* EAI:ok */
 	static char ubuf[MAXNAME_I + 2];
+#if _FFR_8BITENVADDR
+	int len;
+#endif
 
 	if (tTd(24, 5))
 	{
@@ -2212,7 +2215,12 @@ badaddr:
 	}
 
 	/* rewrite according recipient mailer rewriting rules */
-	macdefine(&e->e_macro, A_PERM, 'h', a->q_host);
+#if _FFR_8BITENVADDR
+	p = quote_internal_chars(a->q_host, NULL, &len, NULL);
+#else
+	p = a->q_host;
+#endif
+	macdefine(&e->e_macro, A_PERM, 'h', p);
 
 	if (ConfigLevel >= 10 ||
 	    !bitset(RF_SENDERADDR|RF_HEADERADDR, flags))
@@ -2478,7 +2486,6 @@ static struct qflags	AddressFlags[] =
 	{ "QEXPANDED",		QEXPANDED	},
 	{ "QDELIVERED",		QDELIVERED	},
 	{ "QDELAYED",		QDELAYED	},
-	{ "QTHISPASS",		QTHISPASS	},
 	{ "QALIAS",		QALIAS		},
 	{ "QBYTRACE",		QBYTRACE	},
 	{ "QBYNDELAY",		QBYNDELAY	},
@@ -2486,9 +2493,10 @@ static struct qflags	AddressFlags[] =
 	{ "QINTBCC",		QINTBCC		},
 	{ "QDYNMAILER",		QDYNMAILER	},
 	{ "QSECURE",		QSECURE		},
+	{ "QQUEUED",		QQUEUED		},
+	{ "QINTREPLY",		QINTREPLY	},
 	{ "QTHISPASS",		QTHISPASS	},
 	{ "QRCPTOK",		QRCPTOK		},
-	{ "QQUEUED",		QQUEUED		},
 	{ NULL,			0		}
 };
 
@@ -2881,7 +2889,11 @@ maplocaluser(a, sendq, aliaslevel, e)
 {
 	register char **pvp;
 	register ADDRESS *SM_NONVOLATILE a1 = NULL;
+	char *p;
 	char pvpbuf[PSBUFSIZE];
+#if _FFR_8BITENVADDR
+	int len;
+#endif
 
 	if (tTd(29, 1))
 	{
@@ -2898,7 +2910,12 @@ maplocaluser(a, sendq, aliaslevel, e)
 		return;
 	}
 
-	macdefine(&e->e_macro, A_PERM, 'h', a->q_host);
+#if _FFR_8BITENVADDR
+	p = quote_internal_chars(a->q_host, NULL, &len, NULL);
+#else
+	p = a->q_host;
+#endif
+	macdefine(&e->e_macro, A_PERM, 'h', p);
 	macdefine(&e->e_macro, A_PERM, 'u', a->q_user);
 	macdefine(&e->e_macro, A_PERM, 'z', a->q_home);
 
@@ -3322,8 +3339,10 @@ rscheck(rwset, p1, p2, e, flags, logl, host, logid, addr, addrstr)
 					  rwset, p1, lbuf, ubuf);
 			else
 				sm_syslog(LOG_NOTICE, logid,
-					  "ruleset=%s, arg1=%s%s, reject=%s",
-					  rwset, p1, lbuf, MsgBuf);
+					  "ruleset=%s, arg1=%s%s, %s=%s",
+					  rwset, p1, lbuf,
+					  bitset(RSF_STATUS, flags) ? "status" : "reject",
+					  MsgBuf);
 		}
 
 	 finis: ;
